package com.example.e_systemmobile;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.activity.OnBackPressedCallback;
import androidx.viewpager2.widget.ViewPager2;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.example.e_systemmobile.adapters.SingleCourseOverviewAdapter;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.android.material.tabs.TabLayout;
import com.google.android.material.tabs.TabLayoutMediator;

public class CourseDetail extends Fragment {

    private TabLayout mTabLayoutCourseOverview;
    private ViewPager2 mViewPager2CourseOverview;
    private ActionBar actionBar;
    private final String[] TABS_TITLE = new String[]{"Course", "Participants", "Attendance", "Grades"};

    public CourseDetail() {

    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        actionBar = ((AppCompatActivity)getActivity()).getSupportActionBar();
        // Add Back button
        actionBar.setDisplayHomeAsUpEnabled(true);
        actionBar.setTitle(getArguments().getString(ConstantVariable.COURSE_NAME).toUpperCase());
        // Remove Qr button and Bottom navigation
        getActivity().findViewById(R.id.button_qrcode_scan).setVisibility(View.GONE);
        getActivity().findViewById(R.id.nav_view).setVisibility(View.GONE);

        return inflater.inflate(R.layout.fragment_course_detail, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        mTabLayoutCourseOverview = getActivity().findViewById(R.id.tab_layout_course_detail);
        mViewPager2CourseOverview = getActivity().findViewById(R.id.viewer_course_detail);

        SingleCourseOverviewAdapter singleCourseOverviewAdapter = new SingleCourseOverviewAdapter(getActivity().getSupportFragmentManager(), getLifecycle());
        singleCourseOverviewAdapter.setCourseId(getArguments().getString(ConstantVariable.COURSE_Id));

        mViewPager2CourseOverview.setAdapter(singleCourseOverviewAdapter);

        new TabLayoutMediator(mTabLayoutCourseOverview, mViewPager2CourseOverview, new TabLayoutMediator.TabConfigurationStrategy() {
            @Override
            public void onConfigureTab(@NonNull TabLayout.Tab tab, int position) {
                tab.setText(TABS_TITLE[position]);

            }
        }).attach();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.d("check_fragment","ON_DESTROY");
        // Add Qr button and Bottom navigation when the fragment destroy();
        getActivity().findViewById(R.id.nav_view).setVisibility(View.VISIBLE);
        getActivity().findViewById(R.id.button_qrcode_scan).setVisibility(View.VISIBLE);
    }
}