package com.example.e_systemmobile;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.widget.EditText;
import android.widget.Toast;

import com.example.e_systemmobile.databinding.ActivityLoginBinding;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.example.e_systemmobile.helperutils.FormValidation;
import com.example.e_systemmobile.models.ResAuthObj;
import com.example.e_systemmobile.models.UserAuth;
import com.example.e_systemmobile.networkutils.ApiUtils;
import com.example.e_systemmobile.networkutils.RESTService;


import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;


public class Login extends AppCompatActivity {

    private ActivityLoginBinding binding;
    RESTService mRestService;
    SharedPreferences mPreferences;
    public final String Shared_Preferences = ConstantVariable.SHARED_PREFERENCES;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityLoginBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        mRestService = ApiUtils.getRESTService();

        mPreferences = getApplicationContext().getSharedPreferences(Shared_Preferences, MODE_PRIVATE);
        if (mPreferences.getString("Token", "").isEmpty()) {
            binding.buttonLogin.setOnClickListener(v -> {
                String email = binding.editTextEmail.getText().toString().trim();
                String password = binding.editTextPassword.getText().toString().trim();
                if (FormValidation.loginValidation(getApplicationContext(), email, password)) {
                    doLogin(email, password, binding.editTextEmail, binding.editTextPassword);
                }
            });
        } else {
            Intent intent = new Intent(Login.this, MainActivity.class);
            startActivity(intent);
            finish();
        }
    }

    private void doLogin(String email, String password, EditText emailInput, EditText passwordInput) {
        UserAuth userAuth = new UserAuth(email, password);
        Call<ResAuthObj> call = mRestService.login(userAuth);
        call.enqueue(new Callback<ResAuthObj>() {
            @Override
            public void onResponse(Call<ResAuthObj> call, Response<ResAuthObj> response) {
                if (response.isSuccessful()) {
                    ResAuthObj resAuthObj = response.body();
                    if (!resAuthObj.getToken().isEmpty()) {
                        Intent intent = new Intent(Login.this, MainActivity.class);
                        intent.putExtra("Token", resAuthObj.getToken());
                        intent.putExtra("Courses", resAuthObj.getCourses());
                        startActivity(intent);
                        finish();
                    }
                } else {
                    clearInput(emailInput, passwordInput);
                    Toast.makeText(Login.this, "Error! Please try again!", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onFailure(Call<ResAuthObj> call, Throwable t) {
                Toast.makeText(Login.this, t.getMessage(), Toast.LENGTH_SHORT).show();
            }
        });
    }


    private void clearInput(EditText emailInput, EditText passwordInput) {
        emailInput.getText().clear();
        emailInput.requestFocus();
        passwordInput.getText().clear();
    }
}