package com.example.e_systemmobile;

import android.Manifest;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import com.example.e_systemmobile.databinding.ActivityMainBinding;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.example.e_systemmobile.models.Course;
import com.example.e_systemmobile.networkutils.NetworkStateManager;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.lifecycle.Observer;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;

import java.util.ArrayList;

public class MainActivity extends AppCompatActivity {
    public static final String TAG = MainActivity.class.getSimpleName();
    private ActivityMainBinding binding;
    private Intent intent;
    private NavController navController;
    SharedPreferences mPreferences;

    private static final int PERMISSION_CODE_CAMERA_LOCATION = 1;
    public final String Shared_Preferences = ConstantVariable.SHARED_PREFERENCES;
    private String[] mPermissions;

    private ArrayList<Course> mCourses = null;

    private final Observer<Boolean> activeNetworkStateObserver = new Observer<Boolean>() {
        @Override
        public void onChanged(Boolean isConnected) {
            prepareLottieAnimation(isConnected);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        // Saved Token to SharedPreferences
        mPreferences = getSharedPreferences(Shared_Preferences, MODE_PRIVATE);
        if (getIntent().getExtras() != null) {
            if (!getIntent().getStringExtra("Token").isEmpty()) {
                Log.d("text1", getIntent().getStringExtra("Token"));
                mPreferences.edit().putString("Token", getIntent().getStringExtra("Token")).commit();
            }
            if (getIntent().getSerializableExtra("Courses") != null) {
                mCourses = (ArrayList<Course>) getIntent().getSerializableExtra("Courses");
            }
        }

        // Passing each menu ID as a set of Ids because each
        // menu should be considered as top level destinations.
        navController = Navigation.findNavController(this, R.id.nav_host_fragment_activity_main);
        Bundle bundle = new Bundle();
        bundle.putSerializable("Courses", mCourses);
        navController.setGraph(R.navigation.mobile_navigation, bundle);

        AppBarConfiguration appBarConfiguration = new AppBarConfiguration.Builder(R.id.navigation_dashboard, R.id.navigation_notifications, R.id.navigation_more)
                .build();
        NavigationUI.setupActionBarWithNavController(this, navController, appBarConfiguration);
        NavigationUI.setupWithNavController(binding.navView, navController);

        NetworkStateManager.getInstance().getNetworkConnectivityStatus().observe(this, activeNetworkStateObserver);

        binding.buttonQrcodeScan.setOnClickListener(v -> {
            intent = new Intent(getApplicationContext(), QRCodeScannerActivity.class);
            askPermission();
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.account_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_logout:
                mPreferences.edit().remove("Token").commit();
                Intent intentLogout = new Intent(this, Login.class);
                startActivity(intentLogout);
                finish();
                return true;

            case R.id.action_profile:
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onSupportNavigateUp() {
        return navController.navigateUp() || super.onSupportNavigateUp();
    }

    @Override
    protected void onSaveInstanceState(@NonNull Bundle outState) {
        super.onSaveInstanceState(outState);
    }

    private void prepareLottieAnimation(boolean isConnected) {
        Log.d(TAG, "prepareLottieAnimation() called with: isConnected = [" + isConnected + "]");
        if (isConnected) {
            binding.tvNetworkStatus.setVisibility(View.INVISIBLE);
        } else {
            binding.tvNetworkStatus.setVisibility(View.VISIBLE);
            binding.tvNetworkStatus.setText("Disconnected");
            binding.tvNetworkStatus.setBackgroundColor(Color.RED);
        }
    }

    private void askPermission() {
        mPermissions = new String[]{Manifest.permission.CAMERA, Manifest.permission.ACCESS_FINE_LOCATION};
        if (!isPermissionsGranted(this, mPermissions)) {
            ActivityCompat.requestPermissions(this, mPermissions, PERMISSION_CODE_CAMERA_LOCATION);
        } else {
            startActivity(intent);
        }
    }

    private boolean isPermissionsGranted(Context context, String... mPermissions) {
        for (String permission : mPermissions) {
            if (ActivityCompat.checkSelfPermission(context, permission) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case PERMISSION_CODE_CAMERA_LOCATION:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED) {
                    startActivity(intent);
                } else {
                    AlertDialog.Builder builder = new AlertDialog.Builder(this);
                    builder.setTitle(ConstantVariable.Permission_Access_Denied).setMessage(ConstantVariable.Denied_Approved_By_User);
                    builder.setPositiveButton("OK", (dialog, which) -> dialog.cancel());
                    builder.show();
                }
                return;
        }
    }
}