package com.example.e_systemmobile.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.e_systemmobile.R;
import com.example.e_systemmobile.models.Course;

import java.util.ArrayList;

public class CourseOverviewAdapter extends RecyclerView.Adapter<CourseOverviewAdapter.CourseOverviewViewHolder> {
    private ArrayList<Course> mCourses = null;
    private CourseOverviewAdapterOnClick courseOverviewOnClick;

    public interface CourseOverviewAdapterOnClick{
        void courseOnClick(String courseNameClick, String courseId);
    }

    public CourseOverviewAdapter(CourseOverviewAdapterOnClick courseOverviewAdapterOnClick) {
        this.courseOverviewOnClick = courseOverviewAdapterOnClick;
    }

    @NonNull
    @Override
    public CourseOverviewViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        LayoutInflater layoutInflater = LayoutInflater.from(parent.getContext());
        View view = layoutInflater.inflate(R.layout.course_overview_list_item, parent, false);
        return new CourseOverviewViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull CourseOverviewViewHolder holder, int position) {
        holder.mCourseIdTextView.setText(mCourses.get(position).getCourseShortName().toUpperCase());
        holder.mCourseNameTextView.setText(mCourses.get(position).getCourseName().toUpperCase());
        holder.mCategoryNameTextView.setText(mCourses.get(position).getCategory().getCategoryName()
                .toUpperCase());
    }

    @Override
    public int getItemCount() {
        if (mCourses == null) return 0;
        return mCourses.size();
    }

    public void setCoursesData(ArrayList<Course> coursesData){
        mCourses = coursesData;
        notifyDataSetChanged();
    }

    public class CourseOverviewViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener{
        public final TextView mCourseNameTextView;
        public final TextView mCourseIdTextView;
        public final TextView mCategoryNameTextView;

        public CourseOverviewViewHolder(View view) {
            super(view);
            mCourseNameTextView = view.findViewById(R.id.tv_course_name);
            mCourseIdTextView = view.findViewById(R.id.tv_course_id);
            mCategoryNameTextView = view.findViewById(R.id.tv_category_name);
            view.setOnClickListener(this);
        }

        @Override
        public void onClick(View v) {
            courseOverviewOnClick.courseOnClick(mCourses.get(getAdapterPosition()).getCourseName(), mCourses.get(getAdapterPosition()).get_id());
        }
    }
}
