package com.example.e_systemmobile.ui.attendance;

import android.Manifest;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;

import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.example.e_systemmobile.R;
import com.example.e_systemmobile.databinding.FragmentAttendanceBinding;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.example.e_systemmobile.models.QRSession;
import com.example.e_systemmobile.models.SubmittedResponse;
import com.example.e_systemmobile.networkutils.ApiUtils;
import com.example.e_systemmobile.networkutils.RESTService;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.Priority;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.tasks.Task;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class AttendanceFragment extends Fragment implements OnMapReadyCallback {

    private QRSession mQRSession = null;
    private SubmittedResponse mSubmittedResponse;
    private FragmentAttendanceBinding binding;
    private Location mLastLocation;
    private Marker mMarkerLastLocation;
    private GoogleMap mMaps;
    private final Float DEFAULT_ZOOM = 19f;
    private FusedLocationProviderClient fusedLocationProviderClient;
    private RESTService mRestService;
    private SharedPreferences mPreferences;
    private LocationCallback locationCallback;
    private LocationRequest locationRequest;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentAttendanceBinding.inflate(inflater, container, false);
        View view = binding.getRoot();

        // RESTService API
        mRestService = ApiUtils.getRESTService();
        mPreferences = getActivity().getSharedPreferences(ConstantVariable.SHARED_PREFERENCES, Context.MODE_PRIVATE);

        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        // Google Play Services API for getting user's location
        this.fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(getContext());

        // LocationRequest and LocationCall for requestLocationUpdate method in case that getLastLocation() return null;
        locationCallback = new LocationCallback() {
            @Override
            public void onLocationResult(@NonNull LocationResult locationResult) {
                if (locationResult != null && locationResult.getLastLocation() != null) {
                    mLastLocation = locationResult.getLastLocation();
                    fusedLocationProviderClient.removeLocationUpdates(locationCallback);
                }
            }
        };
        locationRequest = new LocationRequest.Builder(Priority.PRIORITY_HIGH_ACCURACY)
                .setIntervalMillis(5000)
                .setMinUpdateIntervalMillis(2000)
                .setMinUpdateIntervalMillis(4).build();
        SupportMapFragment mapFragment = (SupportMapFragment) getChildFragmentManager().findFragmentById(R.id.fragment_map);
        mapFragment.getMapAsync(this);
        verifyUserAndQRSession(mPreferences.getString("Token", ""), getArguments().getString(ConstantVariable.LOCATION_COURSE));

        // Event Click's present
        binding.buttonPresent.setOnClickListener(v -> {
            submitAttendance(mPreferences.getString("Token", ""), mQRSession.getUnitSessionId());
        });
    }

    @Override
    public void onMapReady(@NonNull GoogleMap googleMap) {
        this.mMaps = googleMap;
        if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            mMaps.setMyLocationEnabled(true);

            // Disable Zoom and Scroll
            mMaps.getUiSettings().setZoomGesturesEnabled(false);
            mMaps.getUiSettings().setScrollGesturesEnabled(false);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    // Display the layout and remove loading UI;
    private void finishLoading() {
        binding.layoutSubmitAttendance.setVisibility(View.VISIBLE);
        binding.pbLoadingSubmitAttendance.setVisibility(View.INVISIBLE);
    }

    private void startLoading(){
        binding.layoutSubmitAttendance.setVisibility(View.INVISIBLE);
        binding.pbLoadingSubmitAttendance.setVisibility(View.VISIBLE);
    }

    private void verifyUserAndQRSession(String token, String sessionId) {
        Call<QRSession> call = mRestService.getQRSession(token, sessionId);
        call.enqueue(new Callback<QRSession>() {
            @Override
            public void onResponse(Call<QRSession> call, Response<QRSession> response) {
                if (response.isSuccessful()) {
                    mQRSession = response.body();
                    if (mQRSession != null) {
                        if ((ContextCompat.checkSelfPermission(getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) && isLocationEnabled()) {
                            Task<Location> locationTask = fusedLocationProviderClient.getLastLocation();
                            locationTask.addOnSuccessListener(location -> {
                                if (location != null) {
                                    mLastLocation = location;
                                    float[] result = new float[3];
                                    Location.distanceBetween(mQRSession.getLocation().getLatitude(), mQRSession.getLocation().getLongitude(), mLastLocation.getLatitude(), mLastLocation.getLongitude(), result);
                                    if (result[0] <= Float.parseFloat(mQRSession.getMaxLength()) + ConstantVariable.STANDARD_ERROR_DISTANCE) {
                                        binding.textViewCourseName.setText(mQRSession.getCourseName().toUpperCase());
                                        mMaps.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(mLastLocation.getLatitude(), mLastLocation.getLongitude()), DEFAULT_ZOOM));
                                        finishLoading();
                                    } else {
                                        showAlertDialogBadRequested(getContext(), R.string.invalid_request, R.string.outside_permissible_range, binding.pbLoadingSubmitAttendance);
                                    }
                                } else {
                                    fusedLocationProviderClient.requestLocationUpdates(locationRequest, locationCallback, Looper.myLooper());
                                    requestedStatusAlertDialog(getContext(), R.string.invalid_request, ConstantVariable.UNABLE_LOCATION, binding.pbLoadingSubmitAttendance);
                                }
                            }).addOnFailureListener(e -> {
                                requestedStatusAlertDialog(getContext(), R.string.invalid_request, e.getMessage(), binding.pbLoadingSubmitAttendance);
                            });
                        } else {
                            showAlertDialogBadRequested(getContext(), R.string.invalid_request, R.string.missing_gps_permission_internet, binding.pbLoadingSubmitAttendance);
                        }
                    } else {
                        showAlertDialogBadRequested(getContext(), R.string.invalid_request, R.string.not_enrolled_student, binding.pbLoadingSubmitAttendance);
                    }
                } else {
                    showAlertDialogBadRequested(getContext(), R.string.invalid_request, R.string.not_enrolled_student, binding.pbLoadingSubmitAttendance);
                }
            }

            @Override
            public void onFailure(Call<QRSession> call, Throwable t) {
                Toast.makeText(getContext(), t.getMessage(), Toast.LENGTH_SHORT).show();
                getActivity().finish();
            }
        });
    }

    private void showAlertDialogBadRequested(Context context, int title, int textDisplay, ProgressBar pbLoading) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title).setMessage(textDisplay);
        builder.setPositiveButton(R.string.ok, (dialog, which) -> {
            pbLoading.setVisibility(View.INVISIBLE);
            dialog.cancel();
            getActivity().finish();
        });
        builder.show();
    }

    private void requestedStatusAlertDialog(Context context, int title, String errorMessage, ProgressBar pbLoading) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title).setMessage(errorMessage);
        builder.setPositiveButton(R.string.ok, (dialog, which) -> {
            pbLoading.setVisibility(View.INVISIBLE);
            dialog.cancel();
            getActivity().finish();
        });
        builder.show();
    }

    private boolean isLocationEnabled() {
        LocationManager locationManager = (LocationManager) getContext().getSystemService(Context.LOCATION_SERVICE);
        return locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER) && locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
    }


    private void submitAttendance(String token, String unitSessionId){
        startLoading();
        Call<SubmittedResponse> call = mRestService.getResponseUpdatedAttendance(token, unitSessionId);
        call.enqueue(new Callback<SubmittedResponse>() {
            @Override
            public void onResponse(Call<SubmittedResponse> call, Response<SubmittedResponse> response) {
                if(response.isSuccessful()){
                    mSubmittedResponse = response.body();
                    requestedStatusAlertDialog(getContext(), R.string.success, ConstantVariable.SUCCESS_RESPONSE, binding.pbLoadingSubmitAttendance);
                }else {
                    requestedStatusAlertDialog(getContext(), R.string.failure_updated, ConstantVariable.UNABLE_UPDATE_ATTENDANCE, binding.pbLoadingSubmitAttendance);
                }
            }
            @Override
            public void onFailure(Call<SubmittedResponse> call, Throwable t) {
                Toast.makeText(getContext(), t.getMessage(), Toast.LENGTH_SHORT).show();
                getActivity().finish();
            }
        });
    }
}