package com.example.e_systemmobile.ui.dashboard;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.navigation.Navigation;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.e_systemmobile.R;
import com.example.e_systemmobile.adapters.CourseOverviewAdapter;
import com.example.e_systemmobile.databinding.FragmentDashboardBinding;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.example.e_systemmobile.models.Course;
import com.example.e_systemmobile.networkutils.ApiUtils;
import com.example.e_systemmobile.networkutils.RESTService;

import java.util.ArrayList;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class DashboardFragment extends Fragment implements CourseOverviewAdapter.CourseOverviewAdapterOnClick {
    private FragmentDashboardBinding binding;
    private CourseOverviewAdapter mCourseOverviewAdapter;
    private ArrayList<Course> mCourses = null;
    RESTService mRestService;
    SharedPreferences mPreferences;
    public final String Shared_Preferences = ConstantVariable.SHARED_PREFERENCES;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {

        ((AppCompatActivity) getActivity()).getSupportActionBar().setDisplayHomeAsUpEnabled(false);
        binding = FragmentDashboardBinding.inflate(inflater, container, false);
        View root = binding.getRoot();
        mRestService = ApiUtils.getRESTService();

        mPreferences = getActivity().getSharedPreferences(Shared_Preferences, Context.MODE_PRIVATE);
        // declare adapter
        mCourseOverviewAdapter = new CourseOverviewAdapter(this);

        mCourses = (ArrayList<Course>) getArguments().getSerializable("Courses");
        if (mCourses == null) {
            // ViewModel and LiveData
            DashboardViewModel dashboardViewModel =
                    new DashboardViewModel(mPreferences.getString("Token", ""), mRestService);
            // getCourses(mPreferences.getString("Token", ""));
            dashboardViewModel.getMutableLiveDataCourses().observe(getViewLifecycleOwner(), courses -> {
                mCourseOverviewAdapter.setCoursesData(courses);
            });
        } else {
            mCourseOverviewAdapter.setCoursesData(mCourses);
        }
        // declare view holder
        binding.recycleViewCourseOverview.setLayoutManager(new LinearLayoutManager(getContext(), RecyclerView.VERTICAL, false));
        binding.recycleViewCourseOverview.setHasFixedSize(true);
        binding.recycleViewCourseOverview.setAdapter(mCourseOverviewAdapter);
        return root;
    }

    @Override
    public void courseOnClick(String courseName, String courseId) {
        Bundle bundle = new Bundle();
        bundle.putString(ConstantVariable.COURSE_NAME, courseName);
        bundle.putString(ConstantVariable.COURSE_Id, courseId);
        Navigation.findNavController(this.getActivity(), R.id.nav_host_fragment_activity_main).navigate(R.id.action_navigation_dashboard_to_course_nested_navigation, bundle);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
    }
}