package com.example.e_systemmobile.ui.dashboard.tabattendance;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.example.e_systemmobile.R;
import com.example.e_systemmobile.adapters.AttendancesCourseAdapter;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.example.e_systemmobile.models.Attendance;
import com.example.e_systemmobile.networkutils.ApiUtils;
import com.example.e_systemmobile.networkutils.RESTService;

import java.util.ArrayList;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class TabAttendance extends Fragment {

    private RESTService mRestService;
    private SharedPreferences mPreferences;
    public final String Shared_Preferences = ConstantVariable.SHARED_PREFERENCES;
    private AttendancesCourseAdapter mAttendancesCourseAdapter;
    private RecyclerView mRecyclerViewAttendances;
    private TextView mTVNoDataFound;
    private ProgressBar mPBLoading;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        mRestService = ApiUtils.getRESTService();
        mPreferences = getActivity().getSharedPreferences(Shared_Preferences, Context.MODE_PRIVATE);

        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_tab_attendance, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        mRecyclerViewAttendances = view.findViewById(R.id.recycle_view_attendances);
        mTVNoDataFound = view.findViewById(R.id.tv_list_attendances_message_display);
        mPBLoading = view.findViewById(R.id.pb_loading_attendances);
        mAttendancesCourseAdapter = new AttendancesCourseAdapter();

        getAllAttendances(mPreferences.getString("Token", ""), getArguments().getString("COURSE_ID"));

        mRecyclerViewAttendances.setLayoutManager(new LinearLayoutManager(this.getContext(), RecyclerView.VERTICAL, false));
        mRecyclerViewAttendances.setHasFixedSize(true);
        mRecyclerViewAttendances.setAdapter(mAttendancesCourseAdapter);
    }

    private void getAllAttendances(String token, String courseId){
        Call<ArrayList<Attendance>> call = mRestService.getAttendances(token, courseId);
        mPBLoading.setVisibility(View.VISIBLE);
        call.enqueue(new Callback<ArrayList<Attendance>>() {
            @Override
            public void onResponse(Call<ArrayList<Attendance>> call, Response<ArrayList<Attendance>> response) {
                if(response.isSuccessful()){
                    if(response != null){
                        mPBLoading.setVisibility(View.INVISIBLE);
                        mAttendancesCourseAdapter.setAttendancesData(response.body());
                        return;
                    }
                    else{
                        mPBLoading.setVisibility(View.INVISIBLE);
                        mTVNoDataFound.setVisibility(View.INVISIBLE);
                    }
                }
                else {
                    Toast.makeText(getContext(), "Error! Please try again!", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onFailure(Call<ArrayList<Attendance>> call, Throwable t) {
                Toast.makeText(getContext(), t.getMessage(), Toast.LENGTH_SHORT).show();
            }
        });

    }
}