package com.example.e_systemmobile.ui.dashboard.tabparticipants;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.example.e_systemmobile.R;
import com.example.e_systemmobile.adapters.ParticipantsEnrolledAdapter;
import com.example.e_systemmobile.helperutils.ConstantVariable;
import com.example.e_systemmobile.models.Student;
import com.example.e_systemmobile.networkutils.ApiUtils;
import com.example.e_systemmobile.networkutils.RESTService;

import java.util.ArrayList;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class TabParticipants extends Fragment implements ParticipantsEnrolledAdapter.ParticipantAdapterOnClick{

    private ParticipantsEnrolledAdapter mParticipantsEnrolledAdapter;
    private RecyclerView mRecycleViewParticipants;
    private TextView mTextViewResultParticipants;
    private RESTService mRestService;
    private SharedPreferences mPreferences;
    public final String Shared_Preferences = ConstantVariable.SHARED_PREFERENCES;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        mRestService = ApiUtils.getRESTService();
        mPreferences = getActivity().getSharedPreferences(Shared_Preferences, Context.MODE_PRIVATE);

        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_tab_participants, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        // assigned RecycleView
        mRecycleViewParticipants = view.findViewById(R.id.recycle_view_participants);
        mParticipantsEnrolledAdapter = new ParticipantsEnrolledAdapter(this);

        getAllParticipants(mPreferences.getString("Token", ""), getArguments().getString("COURSE_ID"));

        mRecycleViewParticipants.setLayoutManager(new LinearLayoutManager(this.getContext(), RecyclerView.VERTICAL, false));
        mRecycleViewParticipants.setHasFixedSize(true);
        mRecycleViewParticipants.setAdapter(mParticipantsEnrolledAdapter);
    }

    private void getAllParticipants(String token, String courseId) {
        Call<ArrayList<Student>> call = mRestService.getParticipants(token, courseId);
        call.enqueue(new Callback<ArrayList<Student>>() {
            @Override
            public void onResponse(Call<ArrayList<Student>> call, Response<ArrayList<Student>> response) {
                if(response.isSuccessful()){
                    if(response != null){
                        mParticipantsEnrolledAdapter.setParticipantsData(response.body());
                        return;
                    }
                }
                else {
                    Toast.makeText(getContext(), "Error! Please try again!", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onFailure(Call<ArrayList<Student>> call, Throwable t) {
                Toast.makeText(getContext(), t.getMessage(), Toast.LENGTH_SHORT).show();
            }
        });
    }

    @Override
    public void studentOnClick(String firstName, String lastName, String studentId) {
        Toast.makeText(getContext(), firstName + " " + lastName, Toast.LENGTH_SHORT).show();
    }
}